/*=========================================================================*/
/*                              LOCKDLL.C                                  */
/*                                                                         */
/*  This DLL attaches itself to the address space of ProgMan.   It sets    */
/*  two thread hooksto monitor ProgMan's messages.                         */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/*               IconLock Version 1.0 Copyright (c) 1999                   */
/*                         Ziff Davis, Inc.							       */
/*                       All rights reserved.							   */
/*        First appeared in PC Magazine, US Edition, August, 1999	       */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/

#include "common.h"
#include <shlguid.h>
#include <shlobj.h>
#include "..\help\cntxmap.h"	//for help file
#define MAIN
#include "lockdll.h"

//Shared data section for this dll
#pragma data_seg(".Shared")
_declspec(dllexport) HHOOK hWndProcHook = 0;
_declspec(dllexport)DWORD dwIconLockThread = 0;
#pragma data_seg()
#pragma comment(linker, "/SECTION:.Shared,RWS")

UINT iShutdownMessage;
HWND  hTaskWnd;
WNDPROC fpShellProc;
WNDPROC fpListViewProc;
WNDPROC fpProgManProc;	//1.1a
HHOOK hKeyboardHook;
WNDPROC fpTaskBarProc;
BOOL bInDialog;
int iLastMenuID;
UINT iIconHelperMessage;
HWND hWndShell;
BOOL bInitDone;	 

//Drop interfaces to remove from the Desktop's window properties
const char SZ_DROP[] = TEXT("OleDropTargetInterface");
const char SZ_ACTIVE_DESKTOP[]  = TEXT("Internet Explorer_Server");


//For enumerating window classes
typedef struct {
	LPCSTR pClassname;
	HWND   hWndFound;
} ENUM_CLASS, *LPENUM_CLASS; 




/****************************/
/*         Prototypes       */
/****************************/
EXPORTED EnumListView( HWND hListView );
void RemoveDropTargets( BOOL bLock );
void OnInitMenu( HWND, WPARAM );
void OnExitMenuLoop( );
void OnMenuSelect( WPARAM wParam, LPARAM lParam );
void DoCommand( int iMenuID );
void OnDisplayChange( );
void OnDestroy( );
void LockCommonDlgs( );
void LockFilesFolders( );
WORD GetLockState( );
void NotifyHelpObjects( );
LRESULT EXPORTED CALLBACK ListViewProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
LRESULT EXPORTED CALLBACK ShellProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

//1.1a
LRESULT EXPORTED CALLBACK ProgManProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

//=====================================================================
//  FUNCTION:		DllMain
//
//  PURPOSE:		Standard Win32 DLL entry point
//
//  PARAMETERS:		standard
//
//  RETURN VALUE:	n/a
//
//  COMMENTS:		Sets global hInstGlobal
//=====================================================================
BOOL WINAPI DllMain( HINSTANCE hinst, ULONG ulReason, PCONTEXT pContext )
{
    UNREFERENCED_PARAMETER( pContext );
	UNREFERENCED_PARAMETER( ulReason );
    hInstGlobal = hinst;

	if (ulReason == DLL_PROCESS_ATTACH )
	{
		DEBUGOUT("LockDll DLL_PROCESS_ATTACH %x %x %x" ,hinst, fpListViewProc, fpShellProc );
	}

	if (ulReason == DLL_PROCESS_DETACH)
	{
		DEBUGOUT("LockDll DLL_PROCESS_DETACH %x %x %x", hinst, fpListViewProc, fpShellProc);
	}

	return( TRUE );
}



//=====================================================================
//  FUNCTION:		GetShellDefView
//
//  PURPOSE:		Finds the SHELLDLL_DefView window under ProgMan
//
//  PARAMETERS:		
//
//  RETURN VALUE:   	
//
//  COMMENTS:		Sets global hWndShell
//=====================================================================
void GetShellDefView( )
{
	HWND hwnd = FindWindow( "Progman", NULL );
	if( hwnd )
	{
		ENUM_CLASS enumclass;

		enumclass.pClassname = "SHELLDLL_DefView";	 
		enumclass.hWndFound = 0;
		EnumChildWindows( hwnd, (WNDENUMPROC)EnumProc, (LPARAM)&enumclass ); 
		hWndShell = enumclass.hWndFound;
	}	
}

//=====================================================================
//  FUNCTION:		HookProgMan
//
//  PURPOSE:		This is the entry point from our ICONLOCK.EXE loader.
//                  It installs two hooks for intercepting messages from
//                  ProgMan.   Note that the hooks are thread hooks and 
//                  not system hooks so that there is no system wide
//                  performance hit.
//
//  PARAMETERS:		thread of our EXE and pointer to handle of hooked shell
//
//  RETURN VALUE:	Nonzero if the hook was created and shell handle.
//
//  COMMENTS:		Sets globals hWndProcHook.  There 
//                  is not much more we can do at this point.  We need
//                  to wait until we receive a callback message 
//                  so that we know we are running in the address space
//                  of ProgMan.
//=====================================================================
EXPORTED HHOOK HookProgMan( DWORD dwThread, HWND* pShellWnd )
{
	//save the thread ID of IconLock.exe
    dwIconLockThread = dwThread;

	GetShellDefView( );
	*pShellWnd = hWndShell;

    if( hWndShell )
	{
		HMODULE hMod = GetModuleHandle( SZ_DLLNAME );
		hWndProcHook = SetWindowsHookEx( WH_GETMESSAGE, CallWndProc, hMod, GetWindowThreadProcessId( hWndShell, NULL ) );
		//send a message to kick things off immediately 
		PostMessage( hWndShell, WM_NULL, 0, 0 );
	}

	return( hWndProcHook );
}


//=====================================================================
//  FUNCTION:		DoInit
//
//  PURPOSE:		Perform various init functions
//                  while in the context of ProgMan.
//
//  PARAMETERS:		n/a
//
//  RETURN VALUE:	n/a
//
//  COMMENTS:		
//=====================================================================
void DoInit( )
{
char szName[MAX_PATH];
LPSTR p;
OSVERSIONINFO osvi;	//1.1a
BOOL bIsWindows95;

	//set flag so we don't do this more than once
    bInitDone = TRUE;

	bWinNT = IsWinNT( );
	bActiveDesktop = IsActiveDesktop( );	//1.1

	//register two private messages
    //iIconHelperMessage is sent or received from LockHlpr.dll
    //to pass the lock state and options
    //iShutdownMessage is sent from IconLock.exe when it is 
    //being unloaded
    iIconHelperMessage = RegisterWindowMessage( SZ_APPNAME );
	iShutdownMessage = RegisterWindowMessage( SZ_SHUTDOWN );

	//get the global hListView -- the ListView control under ProgMan
    GetListViewHandle( );
	
    if( hListView == 0 ) {
        ShowError( IDS_LISTVIEW_ERROR );
		return;
	}

	//Qualify the INI file to the same location as our DLL
	GetModuleFileName( GetModuleHandle( SZ_DLLNAME ), szName, sizeof( szName ) );
	p = strrchr( szName, '\\' );

	if( p )
		*(++p) = 0;
	else 
		p = szName;

	strcat( p, SZ_FILE_INI );
	strcpy( szIniFile, szName );

	//read in our lockdata data file
	ReadDataFile( NULL );

	//Do various IconLock Options
    if( iconlock.byDesktopLocked ) 
	{
		RemoveDropTargets( TRUE );
	}
    
	
	if( iconlock.byDesktopLocked && iconlock.byTaskBarLocked ) 	
	{
		HookTaskBar( TRUE );
	}
	
    NotifyHelpObjects( );
	LockCommonDlgs( );
	LockFilesFolders( );

	//Get the SHELLDLL_DefView in ProgMan, set global hWndShell	 
    GetShellDefView( );
	fpShellProc = (WNDPROC)SetWindowLong( hWndShell, GWL_WNDPROC, (LONG)ShellProc );
	fpListViewProc = (WNDPROC)SetWindowLong( hListView, GWL_WNDPROC, (LONG)ListViewProc );

	//1.1a -- Win95 only sends the WM_DISPLAYCHANGE message to top level windows
	//we need to subclass the ProgMan window in order implement the auto restore feature
	osvi.dwOSVersionInfoSize = sizeof( osvi );
	GetVersionEx( &osvi );
	bIsWindows95 = (osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) &&
 				   ( (osvi.dwMajorVersion == 4) && (osvi.dwMinorVersion == 0) );
	if( bIsWindows95 )
		fpProgManProc = (WNDPROC)SetWindowLong( FindWindow( "Progman", NULL ), GWL_WNDPROC, (LONG)ProgManProc );


    //You must set the keyboard hook while in the context of ProgMan
    //Otherwise your dialog boxes, as well as other things, will crash
	hKeyboardHook = SetWindowsHookEx( WH_KEYBOARD, KeyboardProc, hInstGlobal, GetCurrentThreadId() );

	//Display the Options dialog for first time installs
	PostMessage( hWndShell, WM_NULL, 0, 0 );
				  
}

//=====================================================================
//  FUNCTION:   LockFileFolders		
//
//  PURPOSE:	Places a file lock on the files comprising the destop	
//              file system
//  PARAMETERS:		
//
//  RETURN VALUE:	
//
//  COMMENTS:   This is only done if the user has selected the
//              'Lock Desktop files and Folders' option.		
//=====================================================================
void LockFilesFolders( )
{
	BOOL bLock = ( (GetLockState( ) & FILESFOLDERS_LOCKED ) == FILESFOLDERS_LOCKED );

	if( bLock ) 
		LockFiles( );
	else
		UnlockDesktopFiles( );
}

//=====================================================================
//  FUNCTION:		LockCommonDlgs
//
//  PURPOSE:        Adds a virtual folder to the Desktop namespace		
//                  It also sets the ShellFolder key for our LockDlgs.Dll
//                  server to 'CallForAttributes'.  This will cause 
//                  the shell to query our server whenver the desktop
//                  is accessed.
//
//  PARAMETERS:		
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//=====================================================================
void LockCommonDlgs( )
{
	static char SZ_LOCKDLGKEY[] = {"Software\\Microsoft\\Windows\\CurrentVersion\\explorer\\Desktop\\NameSpace\\{7248D1B1-159D-11D3-A17F-00A0C90AE44B}" };
	static char SZ_SHELLFOLDER[]= {"CLSID\\{7248D1B1-159D-11D3-A17F-00A0C90AE44B}\\ShellFolder" };
	//from our LockDlg.dll


	BOOL bLock = ( (GetLockState( ) & COMMONDLGS_LOCKED) == COMMONDLGS_LOCKED );
	
    if( !bLock )    //remove the keys
	{
		RecursiveRegDeleteKey( HKEY_LOCAL_MACHINE, SZ_LOCKDLGKEY  );
		RecursiveRegDeleteKey( HKEY_CLASSES_ROOT , SZ_SHELLFOLDER );

	} else {        //add the keys
		DWORD dwDisp;
		HKEY hKey;
		LONG lResult = RegCreateKeyEx( HKEY_LOCAL_MACHINE, 
										SZ_LOCKDLGKEY, 
										0, 
										NULL, 
										REG_OPTION_NON_VOLATILE, 
										KEY_WRITE, 
										NULL, 
										&hKey, 
										&dwDisp); 

		if( lResult == ERROR_SUCCESS )
			RegCloseKey( hKey );

		lResult = RegCreateKeyEx( HKEY_CLASSES_ROOT, 
										SZ_SHELLFOLDER, 
										0, 
										NULL, 
										REG_OPTION_NON_VOLATILE, 
										KEY_WRITE, 
										NULL, 
										&hKey, 
										&dwDisp); 

		//Add the CallForAttributes key under our ShellFolder entry
        if( lResult == ERROR_SUCCESS )
		{
			char szData[] = "";

			lResult = RegSetValueEx(   hKey, 
									 "CallForAttributes", 
									 0, 
									 REG_SZ, 
									 (LPBYTE)szData, 
									 lstrlen(szData) + 1); 

			RegCloseKey( hKey );
			
		}
	}
}



//=====================================================================
//  FUNCTION:	GetLockState
//
//  PURPOSE:    Returns the various option flags and the locked or 
//              unlocked state of the desktop		
//
//  PARAMETERS:		
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//=====================================================================
WORD GetLockState( )
{
	WORD iState = 0;	
	
	if( iconlock.byDesktopLocked    ) iState = DESKTOP_LOCKED;
	if( iconlock.byExplorerLocked   ) iState |= LOCK_EXPLORER;
	if( iconlock.byLockCommonDlgs   ) iState |= LOCK_COMMONDLGS;
	if( iconlock.byLockFilesFolders ) iState |= LOCK_FILESFOLDERS;

	return( iState );
}


//=====================================================================
//  FUNCTION:	NotifyHelpObjects	
//
//  PURPOSE:	Broadcasts a private message with the current lock
//              state.  The LockHlpr.dll will pick this up and
//              enable / disable locking within Explorer.
//
//  PARAMETERS:		
//
//  RETURN VALUE:	
//
//  COMMENTS:	This is done so that the user won't have to close
//              and reopen Explorer windows everytime they lock and
//              unlock the desktop.	
//=====================================================================
void NotifyHelpObjects( BOOL bLock )	
{
	SendMessage( HWND_BROADCAST, iIconHelperMessage, (WPARAM)GetLockState( ), 0 );
	return;
}



//=====================================================================
//  FUNCTION:		EnumProc
//
//  PURPOSE:		Searches the child windows of ProgMan for the
//					window class in lParam
//
//  PARAMETERS:		hWnd -- handle to ProgMan window
//
//  RETURN VALUE:	TRUE to continue enumeration
//
//  COMMENTS:		Sets global hListView.  Called by GetListViewHandle
//=====================================================================
BOOL CALLBACK EnumProc( HWND hWnd, LPARAM lParam )
{
char sz[256];
LPENUM_CLASS pEnumClass = (LPENUM_CLASS) lParam;
 	
	GetClassName( hWnd, sz, sizeof(sz) );

	if( strcmp( sz, pEnumClass->pClassname ) == 0 ) 
	{
		pEnumClass->hWndFound = hWnd;
		return( FALSE );
	}

	return( TRUE );
}



//=====================================================================
//  FUNCTION:		GetListViewHandle
//
//  PURPOSE:		Finds ProgMan class and enumerates the child windows
//
//  PARAMETERS:		n/a
//
//  RETURN VALUE:	n/a
//
//  COMMENTS:		Gets global hListView for the SysListView32 control
//=====================================================================
void GetListViewHandle( )
{
	if( hListView == 0 )
	{
		HWND hwnd = FindWindow( "Progman", NULL );
		if( hwnd )
		{
			ENUM_CLASS enumclass;

			enumclass.pClassname = WC_LISTVIEW;	//SysListView32 
			enumclass.hWndFound = 0;
			EnumChildWindows( hwnd, (WNDENUMPROC)EnumProc, (LPARAM)&enumclass ); 
			hListView = enumclass.hWndFound;
		}
		
	}
}



//=====================================================================
//  FUNCTION:		VerifyPassword
//
//  PURPOSE:		Reads our INI file and gets global bLockDesktop 
//                  If TRUE then the desktop is locked.
//
//  PARAMETERS:		bLock -- if TRUE then the user has selected
//                  to lock the desktop. FALSE is to Unlock the
//                  desktop.
//
//  RETURN VALUE:	returns TRUE if password is created / correct
//
//  COMMENTS:		
//=====================================================================
BOOL VerifyPassword( BOOL bLock )
{
char szPassword[MAX_PASSWORD_LEN];

	//if trying to lock the desktop then check if a password has been entered
	if( bLock )
	{
		Decrypt( szPassword, &iconlock.Password );
		if( szPassword[0] == 0 )
			return TRUE;
	}
	return( DialogBox(hInstGlobal, MAKEINTRESOURCE( IDD_ENTER_PASSWORD_DLG ), GetDesktopWindow( ), EnterPassWordProc ) );
}



//=====================================================================
//  FUNCTION:		IsAutoArrangeEnabled
//
//  PURPOSE:		Checks if the ListView control has auto arrange set. 
//                  If so, we can't change the icon layout.
//
//  PARAMETERS:		none
//
//  RETURN VALUE:	Returns true if LVS_AUTOARRANGE is set.  An
//                  error message will be displayed to the user.
//
//  COMMENTS:       We can't change this value programmatically
//                  since it causes Explorer.exe to stack fault.
//                  There are ways, but they differ between shell versions
//=====================================================================
BOOL IsAutoArrangeEnabled( )
{
	if( hListView )
		return( GetWindowLong( hListView, GWL_STYLE ) & LVS_AUTOARRANGE );

	return( FALSE );
}


//=====================================================================
//  FUNCTION:		SaveIconLocations
//
//  PURPOSE:		Saves the x,y positions of the icons on the desktop
//                  to our INI file.
//
//  PARAMETERS:		pSection -- the section name to save under.  This
//                  could be a custom entry or the desktop resolution,
//                  i.e.  800x600
//
//  RETURN VALUE:	none
//
//  COMMENTS:       
//=====================================================================
void SaveIconLocations( LPCSTR pSection )
{
char szName[MAX_PATH],szTmp[MAX_PATH];
char szPoints[30];
POINT pt;
int index = -1;

	if( hListView )
	{
		//purge the ini file section
		MyWritePrivateProfileString( pSection, NULL, NULL, szIniFile );

		//save the screen dimensions
		wsprintf( szTmp, "%ux%u",GetSystemMetrics( SM_CXSCREEN ), 
  								 GetSystemMetrics( SM_CYSCREEN ) ); 

		MyWritePrivateProfileString( pSection, SZ_DIMENSIONKEY, szTmp, szIniFile );	

		while( TRUE )
		{
			index = ListView_GetNextItem( hListView, index, LVNI_ALL );
	
    		if( index < 0 )
				break;
	
    		ListView_GetItemText( hListView, index, 0, szName, sizeof( szName ) );
			ListView_GetItemPosition( hListView, index, &pt );
			wsprintf( szPoints, "%u,%u", pt.x, pt.y );

            //check to see if a duplicate icon name exists
            //this can happen if the user had a Folder or a shell entry 
            //and a desktop item with the same name
            if( MyGetPrivateProfileString( pSection, szName, "", szTmp, sizeof( szTmp ), szIniFile ) )
                ShowError( IDS_DUPLICATE_ICON, szName );
            else 
			    MyWritePrivateProfileString( pSection, szName, szPoints, szIniFile );

		}
	} 
	//1.1a flush the ini file
	MyWritePrivateProfileString( NULL, NULL, NULL, szIniFile );
}

//=====================================================================
//  FUNCTION:		GetScreenDimensions
//
//  PURPOSE:		fills in the pSectionName with the current width and 
//                  height of the display
//
//  PARAMETERS:		pSectionName -- the section name to save or restore with.
//
//  RETURN VALUE:	none
//
//  COMMENTS:       The dimensions are prefixed with SZ_APPNAME.  For example
//                  IconLock 800x600 or IconLock 640x480.
//                  This allows us to skip these entries when doing a 
//                  restore with a custom name -- Save As... & Restore... 
//=====================================================================
void GetScreenDimensions( LPSTR pSectionName )
{
	wsprintf( pSectionName, "%s %ux%u", SZ_APPNAME,
										GetSystemMetrics( SM_CXSCREEN ), 
  								        GetSystemMetrics( SM_CYSCREEN ) ); 
}

//=====================================================================
//  FUNCTION:   WriteUndoSection		          
//
//  PURPOSE:    Called after the user does a Restore or QuickRestore		                                
//
//  PARAMETERS: The Section name and an index, or current count of the
//              icons on the desktop		                
//
//  RETURN VALUE:	      
//
//  COMMENTS:		                          
//=====================================================================
WriteUndoSection( LPCSTR szName, int index )
{
char szPoints[30];
POINT pt;

	ListView_GetItemPosition( hListView, index, &pt );
	wsprintf( szPoints, "%u,%u", pt.x, pt.y );
	MyWritePrivateProfileString( SZ_UNDO, szName, szPoints, szIniFile );

}


//=====================================================================
//  FUNCTION:		RestoreIconLocations
//
//  PURPOSE:		Restores the positions of all icons found in the 
//                  pSection section of our INI file.
//
//  PARAMETERS:		pSection -- the section name to restore with.
//
//  RETURN VALUE:	none
//
//  COMMENTS:       Some attempt has been made to insure that the 
//                  icon won't be displayed outside of the screen
//                  boundaries.  This could use some more work.
//=====================================================================
void RestoreIconLocations( LPCSTR pSection )
{
char szName[MAX_PATH];
char szPoints[30];
LPSTR p;
int x,y;
int height, width;
DWORD dwSpacing;
int index = -1;
RECT rc;	
POINT pt;

	dwSpacing = ListView_GetItemSpacing( hListView, FALSE );
	SystemParametersInfo( SPI_GETWORKAREA, 0, &rc, FALSE ); 
	width = rc.right - rc.left;
	height = rc.bottom - rc.top;

    //check to see if the user has set the Auto Arrange option.
    //If so, show an error and exit.
	if( IsAutoArrangeEnabled( ) )
	{
		ShowError( IDS_AUTOARRANGE_ERROR );
		return;
	}
	
	if( hListView )
	{
		//purge the Undo section if not doing an undo 
		if( strcmp( pSection, SZ_UNDO ) )
			MyWritePrivateProfileString( SZ_UNDO, NULL, NULL, szIniFile );

		while( TRUE )
		{
			index = ListView_GetNextItem( hListView, index, LVNI_ALL );
			if( index < 0 )
				break;

			ListView_GetItemText( hListView, index, 0, szName, sizeof( szName ) );
			
			//if not doing an undo 
			if( strcmp( pSection, SZ_UNDO ) )
				WriteUndoSection( szName, index );
			
			if( MyGetPrivateProfileString( pSection, szName, "", szPoints, sizeof( szPoints ), szIniFile) )
			{
				x = atoi( szPoints );
				p = strchr( szPoints, ',' );
				if( p )
				{
					y = atoi( p+1 );

					// check coordinates for screen bounds
					if( x < 0 ) x = 0;
					if( y < 0 ) y = 0;
					if( x > width )
						x = width - dwSpacing;
					if( y > height )
						y = height - dwSpacing;

					//only update if different than current location -- to avoid flicker
					ListView_GetItemPosition( hListView, index, &pt );
					if( pt.x != x || pt.y != y ) 
						ListView_SetItemPosition( hListView, index, x, y );
				}
			}

		}

	} 

}


//=====================================================================
//  FUNCTION:   FindActiveDesktop		
//
//  PURPOSE:	Searches ProgMan for the Active Desktop window	
//
//  PARAMETERS: Handle to ProgMan	    	
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//=====================================================================
HWND FindActiveDesktop( HWND hwnd )
{
	ENUM_CLASS enumclass;

	enumclass.pClassname = SZ_ACTIVE_DESKTOP;	
	enumclass.hWndFound = 0;
	EnumChildWindows( hwnd, (WNDENUMPROC)EnumProc, (LPARAM)&enumclass ); 
	return( enumclass.hWndFound );
}



//=====================================================================
//  FUNCTION:   RemoveDropTargets		          
//
//  PURPOSE:    Removes the window properties of the Desktop for OLE		                                
//              Drop.
//  PARAMETERS:	Flag to remove or restore the properties	                
//
//  RETURN VALUE:	      
//
//  COMMENTS:	Although this disables OLE Drop from the window,
//              it is done through a non-programatic interface.  I
//              wouldn't recommend this as a general technique for
//              disabling OLE Drop.   However, since there is only
//              one Desktop window ( at least in Win9x ) and it 
//              exists for the life of the session, we can probably
//              get away with this.
//              
//=====================================================================
void RemoveDropTargets( BOOL bLock )
{
static HANDLE hDropNormalDesktop = 0;
static HANDLE hDropActiveDesktop = 0;

	HWND hProgman = FindWindow( "Progman", NULL );	
	HWND hActiveDesktop = FindActiveDesktop( hProgman );
	
	if( bLock ) //remove Drop interface
	{
		if( hProgman && ( hDropNormalDesktop == 0 ) )
		{
			hDropNormalDesktop = GetProp( hProgman, SZ_DROP );
			RemoveProp( hProgman, SZ_DROP );
		}

		if( hActiveDesktop && ( hDropActiveDesktop == 0) )
		{
			hDropActiveDesktop = GetProp( hActiveDesktop, SZ_DROP );
			RemoveProp( hActiveDesktop, SZ_DROP );
		}

	} else {   //restore Drop interface
 		if( hProgman && hDropNormalDesktop ) 	
		{
			SetProp( hProgman, SZ_DROP, hDropNormalDesktop );
			hDropNormalDesktop = 0;
		}

 		if( hActiveDesktop && hDropActiveDesktop ) 	
		{
			SetProp( hActiveDesktop, SZ_DROP, hDropActiveDesktop );
			hDropActiveDesktop = 0;
		}

	}

}




//=====================================================================
//  FUNCTION:		CallWndProc
//
//  PURPOSE:		This is the callback procedure for our WH_CALLWNDPROC
//                  hook.   It simply waits for the first message and 
//                  then calls the DoInit( ) function.  You cannot 
//                  remove the hook however since our DLL would then
//                  be unloaded.
//
//  PARAMETERS:		standard
//
//  RETURN VALUE:	standard
//
//  COMMENTS:		
//=====================================================================
LRESULT EXPORTED CALLBACK CallWndProc( int nCode, WPARAM wParam, LPARAM lParam  )
{
HHOOK hHook = hWndProcHook;

	//can only get the true handle(s) while in the context, address space, of ProgMan
	if( !bInitDone ) 
	{
		DoInit( );
	}

    if( hHook)
       return( CallNextHookEx(  hHook, nCode, wParam,  lParam  ) );
  
	return( 0 );
} 



//=====================================================================
//  FUNCTION:	    ListViewProc	
//
//  PURPOSE:        Processes various messages from ProgMan's ListView		
//
//  PARAMETERS:		
//
//  RETURN VALUE:	
//

//  COMMENTS:       This is where we watch for the Desktop context menu
//                  being created so we can attach our own to it.  Also,
//                  if the desktop is 'locked' we check for ListView
//                  notifications.
//=====================================================================
LRESULT EXPORTED CALLBACK ListViewProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
		switch( uMsg )
		{		
			case WM_INITMENU:
				OnInitMenu( hWnd, wParam );
				break;

            case WM_MENUSELECT:
				OnMenuSelect( wParam, lParam );
				return 0;
				break;
			
			case WM_EXITMENULOOP:
				OnExitMenuLoop( );
				return 0;
				break;

			case WM_DESTROY:
				OnDestroy( );
				return 0;
				break;

	}

		if( fpListViewProc )
			return CallWindowProc( fpListViewProc, hWnd, uMsg, wParam, lParam );
		return( 0 );


}


//=====================================================================
//  FUNCTION:   ProgManProc		 1.1a
//
//  PURPOSE:	Subclasses the Progman window
//
//  PARAMETERS:		
//
//  RETURN VALUE:	
//
//  COMMENTS:   Required for Win95 in order to support the WM_DISPLAYCHANGE
//              message.  Win95 only sends this message to top level
//              windows.
//=====================================================================
LRESULT EXPORTED CALLBACK ProgManProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	if( uMsg == WM_DISPLAYCHANGE )
		OnDisplayChange( );

	if( fpProgManProc )
			return CallWindowProc( fpProgManProc, hWnd, uMsg, wParam, lParam );

	return( 0 );

}		


//=====================================================================
//  FUNCTION:   ShellProc		
//
//  PURPOSE:	Processes various messages from ProgMan's SHELLDLL_DefView	
//
//  PARAMETERS:		
//
//  RETURN VALUE:	
//
//  COMMENTS:   This was required for NT, under 9x we could get away with
//              just the ListView Proc.		
//=====================================================================
LRESULT EXPORTED CALLBACK ShellProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
		switch( uMsg )
		{		
			case WM_NOTIFY:
			{
				LPNM_LISTVIEW pnmv = (LPNM_LISTVIEW)lParam;

				if( iconlock.byDesktopLocked )
					if( pnmv->hdr.code == LVN_BEGINDRAG || pnmv->hdr.code == LVN_BEGINRDRAG )
						return 0;
			}
			break;	

			
			case WM_MOUSEMOVE:
			case WM_NCMOUSEMOVE:
				if( wParam )
					return 0;
			break;



			#if 0
			case WM_NULL:				
				//if first time run then show the options dialog
				if( !iconlock.byOptionsDlgShown )
				{
					iconlock.byOptionsDlgShown = 1;
					DoCommand( IDD_OPTIONS );
				}
				
				//when creating an IconLock.bin file for 
				//the setup diskettes, enable the line below,
				//configure the options dialog and save IconLock.Bin
				//to the install diskettes.
				//iconlock.byOptionsDlgShown = 0;
				break;
			#endif

			case WM_INITMENU:
				OnInitMenu( hWnd, wParam );
				break;

			case WM_DESTROY:
				OnDestroy( );
				return 0;
				break;

			case WM_DISPLAYCHANGE:
				OnDisplayChange( );
				break;

            case WM_MENUSELECT:
				OnMenuSelect( wParam, lParam );
				break;
			
			case WM_EXITMENULOOP:
			    CallWindowProc( fpShellProc, hWnd, uMsg, wParam, lParam );
				OnExitMenuLoop( );
				return 0;

			default:
				//private message used by IconLock's DLLs to return
				//the lock state and options
				if( uMsg == iIconHelperMessage )
				{
					return( GetLockState( ) );
				}
				
				//private message used by IconLock.EXE 
				//to indicate that the system is shutting down
				//and we should unsubclass any windows.
				if( uMsg == iShutdownMessage )
				{
					OnDestroy( );
					return 0;
				}
		
		}
		
		if( fpShellProc )
			return CallWindowProc( fpShellProc, hWnd, uMsg, wParam, lParam );

		return( 0 );
}



//=====================================================================
//  FUNCTION:   OnInitMenu		
//
//  PURPOSE:    This is where we attach our menu to ProgMan's context menu.		
//
//  PARAMETERS:		
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//  The WM_INITMENU is sent when a popup or context menu is about to be displayed.
//  We get the handle to the desktop menu and attach ours to the end of it.
//=====================================================================
void OnInitMenu( HWND hwnd, WPARAM wParam )
{				
	HMENU hDesktopMenu = (HMENU)wParam;

	if( bInDialog )
		return;
		
	if( ListView_GetSelectedCount( hListView ) == 0 )
	{		
		hIconLockMenu = CopyMenu( hDesktopMenu );	//save popup menu to global so we can delete it
		SetLockMenuOptions( hIconLockMenu );
		iLastMenuID = 0;							//init global for the selected menu option.		
	}


	if( iconlock.byDesktopLocked ) 
	{
		DisableDesktopMenuItems( hDesktopMenu );
		DisableIconLockMenuItems( hIconLockMenu );
	}
}


//=====================================================================
//  FUNCTION:   OnMenuSelect		
//
//  PURPOSE:		
//
//  PARAMETERS:		
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//  When a menu item is selected a WM_MENUSELECT message is sent.  Since
//  the desktop actually owns the menu, we won't receive the final menu ID 
//  when the user makes a choice.  So, we need to save the active menu selection
//  in the global iLastMenuID.
//=====================================================================
void OnMenuSelect( WPARAM wParam, LPARAM lParam )
{
	//lParam is set to null when the menu is destroyed
	if( lParam )
	{
		//if our popup menu then
		if( (HMENU)lParam == hIconLockMenu ) 
		{
			iLastMenuID = LOWORD( wParam );
		} else {
			iLastMenuID = 0;
		}
	}
}


//=====================================================================
//  FUNCTION:   OnExitMenuLoop		
//
//  PURPOSE:		
//
//  PARAMETERS:		
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//  This message is sent when the menu loop has been exited.  We examine
//  the global iLastMenuID to see if it is one of our menu items and if so
//  call the appropriate function.  Note that the actual menu item IDs are stored
//  in the dwItemData member of the menu.
//=====================================================================
void OnExitMenuLoop( )
{
	MENUITEMINFO info;

	if( hIconLockMenu == 0 )
		return;
	
	info.cbSize = sizeof( MENUITEMINFO );
	info.fMask = MIIM_DATA | MIIM_ID;

	if( GetMenuItemInfo( hIconLockMenu, iLastMenuID, FALSE, &info ) )
		iLastMenuID = info.dwItemData;	//retrieve the menu id 
	else
		return;
	
	//delete our popup menu -- this is probably not necessary
	DestroyMenu( hIconLockMenu );	
	hIconLockMenu = 0;
	DoCommand( iLastMenuID );
}				


//=====================================================================
//  FUNCTION:	DoCommand	
//
//  PURPOSE:	Perform IconLock menu options    	
//
//  PARAMETERS:		
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//=====================================================================
void DoCommand( int iMenuID )
{
char szSection[80];

	//required for Win95
	if( bInDialog )
		return;

	//see if the user selected one of our options
    switch( iMenuID  )
	{
		case IDD_SAVE:
			GetScreenDimensions( szSection );
			SaveIconLocations( szSection );
			break;

		case IDD_RESTORE:
			GetScreenDimensions( szSection );
			RestoreIconLocations( szSection );
			break;
			
		//Toggle -- Lock or Unlock the desktop
		case IDD_LOCKDESKTOP:						
			if( VerifyPassword( !iconlock.byDesktopLocked ) )
			{
				__try {
				
				//toggle current state
				BOOL bLocked = iconlock.byDesktopLocked = !iconlock.byDesktopLocked;
				
				if( iconlock.byDesktopLocked )
				{
					ReadDataFile( NULL );	//force a re-read
					iconlock.byDesktopLocked = bLocked;
				} 
				
				SaveDataFile( &iconlock, hMenuStrings );				
				
				NotifyHelpObjects( );
				HookTaskBar( iconlock.byDesktopLocked && iconlock.byTaskBarLocked );
				LockCommonDlgs( );
				LockFilesFolders( );
				RemoveDropTargets( iconlock.byDesktopLocked );

				//data files are automatically locked when accessed
				if( !iconlock.byDesktopLocked )
					UnLockDataFiles( );

				} __except( 0 ) {}

				
			}
			break;

		case IDD_OPTIONS:
			bInDialog = TRUE;
			if( DialogBox( hInstGlobal, MAKEINTRESOURCE( IDD_ENTER_PASSWORD_DLG ), GetDesktopWindow( ), EnterPassWordProc ) )
			{
				if( ShowOptionsDlg( hListView ) ) 
				{
					if( iconlock.byDesktopLocked )
					{
						ReadDataFile( NULL );	//force a re-read

						if( iconlock.byExplorerLocked == 0 )
							NotifyHelpObjects( );
						
						HookTaskBar( iconlock.byTaskBarLocked );
						LockCommonDlgs( );
						UnlockDesktopFiles( );
						LockFilesFolders( );
						if( !iconlock.byDesktopLocked )
							UnLockDataFiles( );

					}
				}
			}
			break;

		case IDD_SAVEAS:
			bInDialog = TRUE;
			DialogBox( hInstGlobal, MAKEINTRESOURCE( IDD_SAVEAS_DLG ), GetDesktopWindow( ), SaveAsProc );
			break;
		
		case IDD_RESTORE_CUSTOM:
			bInDialog = TRUE;
			DialogBox( hInstGlobal, MAKEINTRESOURCE( IDD_RESTORE_DLG ), GetDesktopWindow( ), RestoreProc );
			break;
		
		case IDD_UNDO:
			RestoreIconLocations( SZ_UNDO );	
			MyWritePrivateProfileString( SZ_UNDO, NULL, NULL, szIniFile );
			break;

		case IDD_HELP:
			{
			char szHelp[MAX_PATH];
			WinHelp( GetDesktopWindow(), GetHelpFile( szHelp ), HELP_CONTEXT, IDH_overview );
			}
			break;

		case IDD_UNINSTALL:
		{
			HKEY hKey;
			BYTE szData[MAX_PATH * 2];
			DWORD dwType, dwSize;
			BOOL bOK = FALSE;

			//Locate our uninstall string 
			if( RegOpenKey( HKEY_LOCAL_MACHINE,	
				TEXT("Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\IconLock"),&hKey ) == ERROR_SUCCESS )
			{
				dwSize = sizeof( szData );
				if( RegQueryValueEx( hKey, "UninstallString", NULL, &dwType, szData, &dwSize ) == ERROR_SUCCESS )
					bOK = TRUE;
				RegCloseKey( hKey );
			}
			
			//and run the uninstall
            if( bOK )
				WinExec( szData, SW_SHOW );
            else
                ShowError( IDS_UNINSTALLREG_ERROR );

		}
		break;

		case IDD_ABOUT:
			#ifdef _DEBUG
			//Holding down the Shift key while clicking on IconLock's about menu option
			//will unload iconlock.exe and this dll -- debug only,used in development
			if( GetKeyState( VK_SHIFT ) < 0 )
			{
				PostThreadMessage( dwIconLockThread, WM_QUIT,1234,0 );
				break;
			}
			#endif

			bInDialog = TRUE;
			DialogBox( hInstGlobal, MAKEINTRESOURCE( IDD_ABOUT_DLG ), GetDesktopWindow( ), AboutProc );
			break;					
	}	

	bInDialog = FALSE;
}




//=====================================================================
//  FUNCTION:	OnDisplayChange
//
//  PURPOSE:	If the user has changed screen resolutions then 
//              try to restore a saved icon layout.	
//
//  PARAMETERS:		
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//=====================================================================
void OnDisplayChange( )
{
char szSection[80];

	if( iconlock.byAutoRestore )
	{
		GetScreenDimensions( szSection );
		RestoreIconLocations( szSection );
	}
}


//=====================================================================
//  FUNCTION:	OnDestroy
//
//  PURPOSE:	Removes the subclassed callbacks	
//
//  PARAMETERS:		
//
//  RETURN VALUE:	
//
//  COMMENTS:	Called when IconLock.exe is unloading	
//=====================================================================
void OnDestroy(  )
{
	HookTaskBar( FALSE );
	RemoveDropTargets( FALSE );
	UnLockDataFiles( );

	if( fpShellProc )
		SetWindowLong( hWndShell, GWL_WNDPROC, (LONG)fpShellProc );
	if( fpListViewProc )
		SetWindowLong( hListView, GWL_WNDPROC, (LONG)fpListViewProc );
	if( hKeyboardHook )
		UnhookWindowsHookEx( hKeyboardHook );

	//1.1a
	
	if( fpProgManProc )
		SetWindowLong( FindWindow( "Progman", NULL ), GWL_WNDPROC, (LONG)fpProgManProc );

	//1.1
	if( hMenuStrings )
		GlobalFree( hMenuStrings );
	hMenuStrings = 0;

   return;

}



//=====================================================================
//  FUNCTION:		KeyboardProc
//
//  PURPOSE:		This is the callback procedure for our WH_KEYBOARD
//                  hook.  Check for keystrokes that can modfy     
//                  desktop icons -- ie the user is trying to delete
//                  a desktop icon.   If so, and the destop is locked, 
//                  we return TRUE to remove the message from the queue
//                  and prevent the desktop from receiving it.
//
//  PARAMETERS:		standard
//
//  RETURN VALUE:	True if the key should not be processed
//
//  COMMENTS:       We could also check for other keys such as 
//                  ALT-ENTER etc.
//=====================================================================
LRESULT EXPORTED CALLBACK KeyboardProc( int code, WPARAM wParam, LPARAM lParam )
{
HWND hEdit;
	
	if( code >= 0 )
	{
		//If locked then don't let app process the delete key
    	if( iconlock.byDesktopLocked )
		{
			//the GetActiveWindow( ) checks to see if the user 
			//is in one of IconLock's dialog windows
			if( GetActiveWindow( ) == FindWindow( "Progman", NULL ) )
			{
				switch( wParam )
					case VK_DELETE:
					case VK_CONTROL:
					case VK_APPS:	//app menu
					case VK_F2:
					case 'V':
					case 'X':
						return( TRUE );
			}
			//Don't let the user rename items -- required for NT
			hEdit = ListView_GetEditControl( hListView );
			if( hEdit ) 
			{
		  		SendMessage( hListView, WM_CANCELMODE,0,0 );
				DestroyWindow( hEdit );
			}

		}

	} 

	//forward the keystroke
	if( hKeyboardHook )
		return( CallNextHookEx( hKeyboardHook, code, wParam, lParam ) );
	
	return( 0 );
}
 
 
