/*=========================================================================*/
/*                              LockTbar.c                                 */
/*                                                                         */
/*  Subclass functions for locking the TaskBar                             */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/*               IconLock Version 1.0 Copyright (c) 1999                   */
/*                         Ziff Davis, Inc.							       */
/*                       All rights reserved.							   */
/*        First appeared in PC Magazine, US Edition, August, 1999	       */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include "common.h"
#include "lockdll.h"

HWND hStartButton;
HWND hShellTray;

void DoStartButtonMenu( HWND hWnd );
UINT GetEdge(RECT *pRect);      
extern WNDPROC fpTaskBarProc ;

//=====================================================================
//  FUNCTION:		HookTaskBar		          
//
//  PURPOSE:		Locates and subclasses the task bar
//
//  PARAMETERS:		                
//
//  RETURN VALUE:	      
//
//  COMMENTS:		Also gets the handle to the Start Button
//=====================================================================
void HookTaskBar( BOOL bLock )
{
	HWND hShellTray = FindWindow( "Shell_TrayWnd", NULL );

	if( hShellTray )
	{
		if( bLock )
		{
			//1.1
			if( !fpTaskBarProc )
				fpTaskBarProc = (WNDPROC)SetWindowLong( hShellTray, GWL_WNDPROC, (LONG)TaskBarProc );
			
			//find the start button
			hStartButton = FindWindowEx( hShellTray, NULL, "Button", NULL );
			
		} else {

			//unsubclass it
			if( fpTaskBarProc )
				SetWindowLong( hShellTray, GWL_WNDPROC, (LONG)fpTaskBarProc );

			fpTaskBarProc = 0;
		}

	}

}


//=====================================================================
//  FUNCTION:		TaskBarProc
//
//  PURPOSE:		Callback for the subclassed taskbar window		                                
//					Disable various messages
//  PARAMETERS:		                
//
//  RETURN VALUE:	      
//
//  COMMENTS:		Win9x and NT are very different in the messages
//                  used in the taskbar                          
//=====================================================================
LRESULT EXPORTED CALLBACK TaskBarProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static BOOL bInMenu;	
	
	if( iconlock.byDesktopLocked && iconlock.byTaskBarLocked  )
	{
		switch( uMsg )
		{
			case WM_COMMAND:	//ctrl-esc
				DEBUGOUT( "WM_COMMAND %X hi=%X ID=%x inmenu = %x", wParam, HIWORD( wParam ),LOWORD(wParam), bInMenu );

				//1.1 corrected problem where a user could press the Windows key twice to
				//display the Start menu.
				if( bInMenu )
					return 0;

				if( HIWORD(wParam) == BN_CLICKED && lParam == (LPARAM)hStartButton )
				{
					bInMenu = 1; //don't process the start button message twice
					DoStartButtonMenu( hWnd );
					DEBUGOUT(  "DoStartButtonMenu ret" );
					SendMessage( hStartButton, BM_SETSTATE, 0, 0 );	//1.1a Win95 requires Send not Post
					bInMenu = 0;
					return 0;
				}
				break;

			//1.1a -- If the user has the Auto Hide taskbar option set then
			//we need to allow it to become visible, but not allow the user to
			//resize or relocate the bar.
			case WM_NCHITTEST:	//nt
			{
				APPBARDATA abd;
				RECT rc;
				int x_min, y_min;

				//check for autohide
				abd.cbSize = sizeof( APPBARDATA );
				if( ( SHAppBarMessage( ABM_GETSTATE, &abd ) & ABS_AUTOHIDE ) == 0 )
					return 0;
		
				//use the dimensions of the start button 
				//to determine the minimum sizes of the appbar
				//you can't use appbar functions to get this size 
				//when it is hidden -- the results vary between OS versions
				GetClientRect( hStartButton, &rc );
				x_min = rc.right  / 2;
				y_min = rc.bottom / 2;

				x_min = max( 4, x_min );
				y_min = max( 4, y_min );

				//get the current size of the appbar
				GetWindowRect( hWnd, &rc );
				
				//get the position of the appbar
				abd.cbSize = sizeof( APPBARDATA );
				SHAppBarMessage( ABM_GETTASKBARPOS, &abd );

				switch( GetEdge( &abd.rc ) )
				{
					case ABE_LEFT:
						if( rc.right >= x_min )
							return 0;
						break;
					case ABE_TOP:
						if( rc.bottom >= y_min )
							return 0;
						break;
					case ABE_BOTTOM:
						if( ( GetSystemMetrics( SM_CYSCREEN ) - rc.top ) >= y_min )
							return 0;
						break;
					case ABE_RIGHT:
						if( ( GetSystemMetrics( SM_CXSCREEN ) - rc.left ) >= x_min )
							return 0;
						break;
				}
			}
			break;

			case WM_PARENTNOTIFY:	//nt
			case WM_MOUSEACTIVATE:
			case WM_SETCURSOR:
			case WM_HOTKEY:
					return 0;

			case WM_NCRBUTTONDOWN:	//NT prevent taskbar properties
				return 0;
			case WM_NCLBUTTONDOWN:	//prevent sizing
				if( hWnd == hShellTray)
					return 0;
				break;

			case WM_MOUSEMOVE:
			case WM_NCMOUSEMOVE:
				if( wParam )
					return 0;
				break;

			//prevent the user from clicking on the toolbar properties menu
			case WM_INITMENU:
				DestroyMenu( (HMENU)wParam );
				return 0;

			case WM_CONTEXTMENU:
				return 0;
		
		}

	}
	return CallWindowProc( fpTaskBarProc, hWnd, uMsg, wParam, lParam );
}



//=====================================================================
//  FUNCTION:		GetEdge		          
//
//  PURPOSE:		Returns the location for our ShutDown menu		                                
//
//  PARAMETERS:			                
//
//  RETURN VALUE:	      
//
//  COMMENTS:		                          
//=====================================================================
UINT GetEdge(RECT *pRect)      
{     
	UINT uEdge = -1;
	
	if (pRect->top == pRect->left && pRect->bottom > pRect->right)      
	{
		uEdge = ABE_LEFT;      
	}
	else if (pRect->top == pRect->left && pRect->bottom < pRect->right)      
	{
		uEdge = ABE_TOP;      
	}      
	else if (pRect->top > pRect->left )      
	{
		uEdge = ABE_BOTTOM;      
	}      
	else      
	{
		uEdge = ABE_RIGHT;      
	}         
	
	return uEdge;      
}

//=====================================================================
//  FUNCTION:	DoStartButtonMenu		          
//
//  PURPOSE:	Creates a popup menu for Shut Down
//
//  PARAMETERS:		                
//
//  RETURN VALUE:	      
//
//  COMMENTS:	This should be owner-draw!!
//=====================================================================
void DoStartButtonMenu( HWND hWnd )
{
	UINT x,y;
	UINT iFlags, iCmd;
	HMENU hmenu;
	LRESULT lState; 


	APPBARDATA abd;
	abd.cbSize = sizeof( APPBARDATA );
	SHAppBarMessage( ABM_GETTASKBARPOS, &abd );
	
	x = 0;
	y = 0;
	iFlags = 0;

	switch( GetEdge( &abd.rc ) )
	{
		case ABE_BOTTOM:
			y = abd.rc.top;
			iFlags = TPM_LEFTALIGN | TPM_BOTTOMALIGN;
			break;

		case ABE_TOP:
			y = abd.rc.bottom;
			iFlags = TPM_LEFTALIGN  | TPM_TOPALIGN;
			break;

		case ABE_LEFT:
			x = abd.rc.right;
			iFlags = TPM_LEFTALIGN | TPM_TOPALIGN;
			break;

		case ABE_RIGHT:
			x = abd.rc.left;
			iFlags = TPM_RIGHTALIGN | TPM_TOPALIGN;
			break;
	}

	hmenu = CreatePopupMenu();
	AppendMenu( hmenu, MF_STRING, 1, "Sh&ut Down" );
	iCmd = TrackPopupMenu( hmenu, iFlags | TPM_RETURNCMD | TPM_NONOTIFY, x,y, 0, hWnd, NULL );
	DestroyMenu( hmenu );
	
	if( iCmd == 1 )
	{
		ShowShutdownDLG( );
	}

	lState = SendMessage( hStartButton, BM_GETSTATE, 0, 0 );
	if( lState & BST_PUSHED )
		SendMessage( hStartButton, BM_SETSTATE, 0, 0 );	 //1.1a Win95 requires Send not Post

   
}


//=====================================================================
//  FUNCTION:		ShowShutdownDLG		          
//
//  PURPOSE:		Calls undocumented shell ordinal to display the
//					shut down dialog -- 60
//  PARAMETERS:		                
//
//  RETURN VALUE:	We don't really care about the return value	      
//					If the user shuts down, we'll find out about
//					it eventually.
//
//  COMMENTS:		1.1 Displaying the dialog was moved to IconLock.exe		                          
//=====================================================================
UINT ShowShutdownDLG( )
{
	
	HWND hWnd = FindWindowEx( FindWindow( "Progman", 0 ), 0,SZ_CLASS_NAME, 0 ); 

	if( hWnd )
		PostMessage( hWnd, MSG_SHUTDOWNDLG, 0, 0 );
	else
		ShowError( IDS_WINDOWERROR );

	return 0;

}

