/////////////////////////////////////////////////////////////////////////////
// HookTreeView -- subclass the treeview control in explorer
//
//    Copyright (c) 1999 Ziff Davis, Inc. All rights reserved
//    First appeared in PC Magazine, US Edition, August, 1999
//                Programmed by Mark Sweeney
//


#include "StdAfx.h"
#include "HookExplorer.h"
#include <commctrl.h>


  
/////////////////////////////////////////////////////////////////////////////
// 
CHookTreeView::CHookTreeView( )
{
	m_hWnd = 0;
	m_OldWndProc  = 0;
}

/////////////////////////////////////////////////////////////////////////////
// 
CHookTreeView::~CHookTreeView( )
{
	UnSubClass( );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CHookTreeView::SubClass( HWND hWnd )
{
	_ASSERT( hWnd );

	UnSubClass( );

	m_hWnd = hWnd;
	m_OldWndProc = reinterpret_cast<WNDPROC>( SetWindowLong( m_hWnd, GWL_WNDPROC,reinterpret_cast<LONG>( WndProc ) ) );
	
}	

/////////////////////////////////////////////////////////////////////////////
// 
void CHookTreeView::UnSubClass( )
{
	if( m_hWnd && m_OldWndProc )
		SetWindowLong( m_hWnd, GWL_WNDPROC, (LONG)m_OldWndProc);
	m_hWnd = 0;
}	


/////////////////////////////////////////////////////////////////////////////
// GetHwnd()
// returns the hwnd of the subclassed window
//
HWND CHookTreeView::GetHwnd()
{
	return( m_hWnd );
}



/////////////////////////////////////////////////////////////////////////////
// WndProc
// callback function for the treeview control
//

LRESULT CALLBACK CHookTreeView::WndProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	CHookExplorer*  pThis = (CHookExplorer*)Get_pThis( );

	if( pThis == NULL ) {
		DEBUGOUT( _T("INVALID PTHIS CHookTreeView") );
		return FALSE;	//disaster
	}


	if( ( pThis->m_bLockFlags & DESKTOP_LOCKED ) )
	{

		//prevent drag and drop
		if( uMsg == TVM_CREATEDRAGIMAGE )
			return 0;

		if( pThis->m_bLocked && ( pThis->m_bLockFlags == EXPLORER_LOCKED ) )
		{
			switch( uMsg )
			{				
				case WM_RBUTTONDOWN:	
				case WM_COMMAND:			
				case WM_RBUTTONDBLCLK:
				case WM_RBUTTONUP:
					return 1;

				case WM_NCMOUSEMOVE:
				case WM_MOUSEMOVE:
					SetCursor( pThis->m_hLockedCursor );
					if( wParam )
						return 1;
					break;

				case WM_SETCURSOR:
					SetCursor( pThis->m_hLockedCursor );
					return 1;

			}
		}

	}

	return( CallWindowProc( pThis->m_TreeView.m_OldWndProc, hwnd, uMsg, wParam, lParam) );
}
